use criterion::{black_box, criterion_group, criterion_main, BenchmarkId, Criterion};
use rusty_numbers::num::Unsigned;
use rusty_numbers::{factorial, fibonacci, it_factorial, mem_fibonacci, rec_fibonacci};

fn bench_factorial(c: &mut Criterion) {
    let mut group = c.benchmark_group("Factorials");

    for i in [0usize, 5, 10, 15, 20, 25, 30, 34].iter() {
        group.bench_with_input(BenchmarkId::new("Recursive naive", i), i, |b, i| {
            b.iter(|| factorial(black_box(*i)))
        });
        group.bench_with_input(BenchmarkId::new("Iterative", i), i, |b, i| {
            b.iter(|| it_factorial(black_box(*i)))
        });
    }
    group.finish();
}

fn bench_fibonacci(c: &mut Criterion) {
    let mut group = c.benchmark_group("Fibonacci");

    for i in [0usize, 10, 20, 30, 40, 50, 70, 93, 140, 186].iter() {
        group.bench_with_input(BenchmarkId::new("Recursive memoized", i), i, |b, i| {
            b.iter(|| mem_fibonacci(black_box(*i)))
        });
        group.bench_with_input(BenchmarkId::new("Iterative", i), i, |b, i| {
            b.iter(|| fibonacci(black_box(*i)))
        });
    }

    group.finish();

    let mut group = c.benchmark_group("Recursive Fibonacci");
    for i in [0usize, 10, 20, 25, 26, 27, 28, 29, 30].iter() {
        group.bench_with_input(BenchmarkId::new("Naive Recursive", i), i, |b, i| {
            b.iter(|| rec_fibonacci(black_box(*i)))
        });
    }
    group.finish();
}

fn bench_gcd(c: &mut Criterion) {
    let mut group = c.benchmark_group("GCD");

    let max = fibonacci(186).unwrap();
    let max_1 = fibonacci(185).unwrap();

    let med = fibonacci(93).unwrap();
    let med_1 = fibonacci(92).unwrap();

    let small = fibonacci(15).unwrap();
    let small_1 = fibonacci(14).unwrap();

    for input in [(small_1, small), (med_1, med), (max_1, max)].iter() {
        let param = format!("{},{}", input.0, input.1);
        group.bench_with_input(BenchmarkId::new("Binary", &param), input, |b, input| {
            b.iter(|| u128::gcd(black_box(input.0), black_box(input.1)))
        });
        group.bench_with_input(BenchmarkId::new("Euclid", &param), input, |b, input| {
            b.iter(|| u128::e_gcd(black_box(input.0), black_box(input.1)))
        });
    }
    group.finish();
}

criterion_group!(benches, bench_factorial, bench_fibonacci, bench_gcd);
criterion_main!(benches);
